@@ -149,6 +149,24 @@ pub trait AggregatorImpl: AggregatorImplExt + ElementImpl {
         self.parent_negotiated_src_caps(aggregator, caps)
     }
 
+    fn propose_allocation(
+        &self,
+        element: &Self::Type,
+        pad: &AggregatorPad,
+        decide_query: Option<gst::query::Allocation<&gst::QueryRef>>,
+        query: gst::query::Allocation<&mut gst::QueryRef>,
+    ) -> Result<(), gst::LoggableError> {
+        self.parent_propose_allocation(element, pad, decide_query, query)
+    }
+
+    fn decide_allocation(
+        &self,
+        element: &Self::Type,
+        query: gst::query::Allocation<&mut gst::QueryRef>,
+    ) -> Result<(), gst::LoggableError> {
+        self.parent_decide_allocation(element, query)
+    }
+
     #[cfg(any(feature = "v1_18", feature = "dox"))]
     #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
     fn negotiate(&self, aggregator: &Self::Type) -> bool {
@@ -267,6 +285,20 @@ pub trait AggregatorImplExt: ObjectSubclass {
         caps: &gst::Caps,
     ) -> Result<(), gst::LoggableError>;
 
+    fn parent_propose_allocation(
+        &self,
+        element: &Self::Type,
+        pad: &AggregatorPad,
+        decide_query: Option<gst::query::Allocation<&gst::QueryRef>>,
+        query: gst::query::Allocation<&mut gst::QueryRef>,
+    ) -> Result<(), gst::LoggableError>;
+
+    fn parent_decide_allocation(
+        &self,
+        element: &Self::Type,
+        query: gst::query::Allocation<&mut gst::QueryRef>,
+    ) -> Result<(), gst::LoggableError>;
+
     #[cfg(any(feature = "v1_18", feature = "dox"))]
     #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
     fn parent_negotiate(&self, aggregator: &Self::Type) -> bool;
@@ -656,6 +688,61 @@ impl<T: AggregatorImpl> AggregatorImplExt for T {
         }
     }
 
+    fn parent_propose_allocation(
+        &self,
+        element: &Self::Type,
+        pad: &AggregatorPad,
+        decide_query: Option<gst::query::Allocation<&gst::QueryRef>>,
+        query: gst::query::Allocation<&mut gst::QueryRef>,
+    ) -> Result<(), gst::LoggableError> {
+        unsafe {
+            let data = Self::type_data();
+            let parent_class = data.as_ref().parent_class() as *mut ffi::GstAggregatorClass;
+            (*parent_class)
+                .propose_allocation
+                .map(|f| {
+                    gst::result_from_gboolean!(
+                        f(
+                            element.unsafe_cast_ref::<Aggregator>().to_glib_none().0,
+                            pad.to_glib_none().0,
+                            decide_query
+                                .as_ref()
+                                .map(|q| q.as_mut_ptr())
+                                .unwrap_or(ptr::null_mut()),
+                            query.as_mut_ptr()
+                        ),
+                        gst::CAT_RUST,
+                        "Parent function `propose_allocation` failed",
+                    )
+                })
+                .unwrap_or(Ok(()))
+        }
+    }
+
+    fn parent_decide_allocation(
+        &self,
+        element: &Self::Type,
+        query: gst::query::Allocation<&mut gst::QueryRef>,
+    ) -> Result<(), gst::LoggableError> {
+        unsafe {
+            let data = Self::type_data();
+            let parent_class = data.as_ref().parent_class() as *mut ffi::GstAggregatorClass;
+            (*parent_class)
+                .decide_allocation
+                .map(|f| {
+                    gst::result_from_gboolean!(
+                        f(
+                            element.unsafe_cast_ref::<Aggregator>().to_glib_none().0,
+                            query.as_mut_ptr(),
+                        ),
+                        gst::CAT_RUST,
+                        "Parent function `decide_allocation` failed",
+                    )
+                })
+                .unwrap_or(Ok(()))
+        }
+    }
+
     #[cfg(any(feature = "v1_18", feature = "dox"))]
     #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
     fn parent_negotiate(&self, aggregator: &Self::Type) -> bool {
@@ -699,7 +786,7 @@ impl<T: AggregatorImpl> AggregatorImplExt for T {
 
 unsafe impl<T: AggregatorImpl> IsSubclassable<T> for Aggregator {
     fn class_init(klass: &mut glib::Class<Self>) {
-        <gst::Element as IsSubclassable<T>>::class_init(klass);
+        Self::parent_class_init::<T>(klass);
         let klass = klass.as_mut();
         klass.flush = Some(aggregator_flush::<T>);
         klass.clip = Some(aggregator_clip::<T>);
@@ -717,6 +804,8 @@ unsafe impl<T: AggregatorImpl> IsSubclassable<T> for Aggregator {
         klass.update_src_caps = Some(aggregator_update_src_caps::<T>);
         klass.fixate_src_caps = Some(aggregator_fixate_src_caps::<T>);
         klass.negotiated_src_caps = Some(aggregator_negotiated_src_caps::<T>);
+        klass.propose_allocation = Some(aggregator_propose_allocation::<T>);
+        klass.decide_allocation = Some(aggregator_decide_allocation::<T>);
         #[cfg(any(feature = "v1_18", feature = "dox"))]
         {
             klass.sink_event_pre_queue = Some(aggregator_sink_event_pre_queue::<T>);
@@ -726,17 +815,13 @@ unsafe impl<T: AggregatorImpl> IsSubclassable<T> for Aggregator {
             klass.finish_buffer_list = Some(aggregator_finish_buffer_list::<T>);
         }
     }
-
-    fn instance_init(instance: &mut glib::subclass::InitializingObject<T>) {
-        <gst::Element as IsSubclassable<T>>::instance_init(instance);
-    }
 }
 
 unsafe extern "C" fn aggregator_flush<T: AggregatorImpl>(
     ptr: *mut ffi::GstAggregator,
 ) -> gst::ffi::GstFlowReturn {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), gst::FlowReturn::Error, {
@@ -751,7 +836,7 @@ unsafe extern "C" fn aggregator_clip<T: AggregatorImpl>(
     buffer: *mut gst::ffi::GstBuffer,
 ) -> *mut gst::ffi::GstBuffer {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     let ret = gst::panic_to_error!(&wrap, imp.panicked(), None, {
@@ -770,7 +855,7 @@ unsafe extern "C" fn aggregator_finish_buffer<T: AggregatorImpl>(
     buffer: *mut gst::ffi::GstBuffer,
 ) -> gst::ffi::GstFlowReturn {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), gst::FlowReturn::Error, {
@@ -787,7 +872,7 @@ unsafe extern "C" fn aggregator_finish_buffer_list<T: AggregatorImpl>(
     buffer_list: *mut gst::ffi::GstBufferList,
 ) -> gst::ffi::GstFlowReturn {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), gst::FlowReturn::Error, {
@@ -803,7 +888,7 @@ unsafe extern "C" fn aggregator_sink_event<T: AggregatorImpl>(
     event: *mut gst::ffi::GstEvent,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(wrap, imp.panicked(), false, {
@@ -824,7 +909,7 @@ unsafe extern "C" fn aggregator_sink_event_pre_queue<T: AggregatorImpl>(
     event: *mut gst::ffi::GstEvent,
 ) -> gst::ffi::GstFlowReturn {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), gst::FlowReturn::Error, {
@@ -844,7 +929,7 @@ unsafe extern "C" fn aggregator_sink_query<T: AggregatorImpl>(
     query: *mut gst::ffi::GstQuery,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -865,7 +950,7 @@ unsafe extern "C" fn aggregator_sink_query_pre_queue<T: AggregatorImpl>(
     query: *mut gst::ffi::GstQuery,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -883,7 +968,7 @@ unsafe extern "C" fn aggregator_src_event<T: AggregatorImpl>(
     event: *mut gst::ffi::GstEvent,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -897,7 +982,7 @@ unsafe extern "C" fn aggregator_src_query<T: AggregatorImpl>(
     query: *mut gst::ffi::GstQuery,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -912,7 +997,7 @@ unsafe extern "C" fn aggregator_src_activate<T: AggregatorImpl>(
     active: glib::ffi::gboolean,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -932,7 +1017,7 @@ unsafe extern "C" fn aggregator_aggregate<T: AggregatorImpl>(
     timeout: glib::ffi::gboolean,
 ) -> gst::ffi::GstFlowReturn {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), gst::FlowReturn::Error, {
@@ -946,7 +1031,7 @@ unsafe extern "C" fn aggregator_start<T: AggregatorImpl>(
     ptr: *mut ffi::GstAggregator,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -965,7 +1050,7 @@ unsafe extern "C" fn aggregator_stop<T: AggregatorImpl>(
     ptr: *mut ffi::GstAggregator,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -984,7 +1069,7 @@ unsafe extern "C" fn aggregator_get_next_time<T: AggregatorImpl>(
     ptr: *mut ffi::GstAggregator,
 ) -> gst::ffi::GstClockTime {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), gst::ClockTime::NONE, {
@@ -1000,7 +1085,7 @@ unsafe extern "C" fn aggregator_create_new_pad<T: AggregatorImpl>(
     caps: *const gst::ffi::GstCaps,
 ) -> *mut ffi::GstAggregatorPad {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), None, {
@@ -1024,7 +1109,7 @@ unsafe extern "C" fn aggregator_update_src_caps<T: AggregatorImpl>(
     res: *mut *mut gst::ffi::GstCaps,
 ) -> gst::ffi::GstFlowReturn {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     *res = ptr::null_mut();
@@ -1046,7 +1131,7 @@ unsafe extern "C" fn aggregator_fixate_src_caps<T: AggregatorImpl>(
     caps: *mut gst::ffi::GstCaps,
 ) -> *mut gst::ffi::GstCaps {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), gst::Caps::new_empty(), {
@@ -1060,7 +1145,7 @@ unsafe extern "C" fn aggregator_negotiated_src_caps<T: AggregatorImpl>(
     caps: *mut gst::ffi::GstCaps,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -1075,13 +1160,76 @@ unsafe extern "C" fn aggregator_negotiated_src_caps<T: AggregatorImpl>(
     .into_glib()
 }
 
+unsafe extern "C" fn aggregator_propose_allocation<T: AggregatorImpl>(
+    ptr: *mut ffi::GstAggregator,
+    pad: *mut ffi::GstAggregatorPad,
+    decide_query: *mut gst::ffi::GstQuery,
+    query: *mut gst::ffi::GstQuery,
+) -> glib::ffi::gboolean {
+    let instance = &*(ptr as *mut T::Instance);
+    let imp = instance.imp();
+    let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
+    let decide_query = if decide_query.is_null() {
+        None
+    } else {
+        match gst::QueryRef::from_ptr(decide_query).view() {
+            gst::QueryView::Allocation(allocation) => Some(allocation),
+            _ => unreachable!(),
+        }
+    };
+    let query = match gst::QueryRef::from_mut_ptr(query).view_mut() {
+        gst::QueryView::Allocation(allocation) => allocation,
+        _ => unreachable!(),
+    };
+
+    gst::panic_to_error!(&wrap, imp.panicked(), false, {
+        match imp.propose_allocation(
+            wrap.unsafe_cast_ref(),
+            &from_glib_borrow(pad),
+            decide_query,
+            query,
+        ) {
+            Ok(()) => true,
+            Err(err) => {
+                err.log_with_object(&*wrap);
+                false
+            }
+        }
+    })
+    .into_glib()
+}
+
+unsafe extern "C" fn aggregator_decide_allocation<T: AggregatorImpl>(
+    ptr: *mut ffi::GstAggregator,
+    query: *mut gst::ffi::GstQuery,
+) -> glib::ffi::gboolean {
+    let instance = &*(ptr as *mut T::Instance);
+    let imp = instance.imp();
+    let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
+    let query = match gst::QueryRef::from_mut_ptr(query).view_mut() {
+        gst::QueryView::Allocation(allocation) => allocation,
+        _ => unreachable!(),
+    };
+
+    gst::panic_to_error!(&wrap, imp.panicked(), false, {
+        match imp.decide_allocation(wrap.unsafe_cast_ref(), query) {
+            Ok(()) => true,
+            Err(err) => {
+                err.log_with_object(&*wrap);
+                false
+            }
+        }
+    })
+    .into_glib()
+}
+
 #[cfg(any(feature = "v1_18", feature = "dox"))]
 #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
 unsafe extern "C" fn aggregator_negotiate<T: AggregatorImpl>(
     ptr: *mut ffi::GstAggregator,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -1097,7 +1245,7 @@ unsafe extern "C" fn aggregator_peek_next_sample<T: AggregatorImpl>(
     pad: *mut ffi::GstAggregatorPad,
 ) -> *mut gst::ffi::GstSample {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Aggregator> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), None, {
