@@ -107,6 +107,14 @@ pub trait BaseSrcImpl: BaseSrcImplExt + ElementImpl {
     fn unlock_stop(&self, element: &Self::Type) -> Result<(), gst::ErrorMessage> {
         self.parent_unlock_stop(element)
     }
+
+    fn decide_allocation(
+        &self,
+        element: &Self::Type,
+        query: gst::query::Allocation<&mut gst::QueryRef>,
+    ) -> Result<(), gst::LoggableError> {
+        self.parent_decide_allocation(element, query)
+    }
 }
 
 pub trait BaseSrcImplExt: ObjectSubclass {
@@ -168,6 +176,12 @@ pub trait BaseSrcImplExt: ObjectSubclass {
     fn parent_unlock(&self, element: &Self::Type) -> Result<(), gst::ErrorMessage>;
 
     fn parent_unlock_stop(&self, element: &Self::Type) -> Result<(), gst::ErrorMessage>;
+
+    fn parent_decide_allocation(
+        &self,
+        element: &Self::Type,
+        query: gst::query::Allocation<&mut gst::QueryRef>,
+    ) -> Result<(), gst::LoggableError>;
 }
 
 impl<T: BaseSrcImpl> BaseSrcImplExt for T {
@@ -569,11 +583,35 @@ impl<T: BaseSrcImpl> BaseSrcImplExt for T {
                 .unwrap_or(Ok(()))
         }
     }
+
+    fn parent_decide_allocation(
+        &self,
+        element: &Self::Type,
+        query: gst::query::Allocation<&mut gst::QueryRef>,
+    ) -> Result<(), gst::LoggableError> {
+        unsafe {
+            let data = Self::type_data();
+            let parent_class = data.as_ref().parent_class() as *mut ffi::GstBaseSrcClass;
+            (*parent_class)
+                .decide_allocation
+                .map(|f| {
+                    gst::result_from_gboolean!(
+                        f(
+                            element.unsafe_cast_ref::<BaseSrc>().to_glib_none().0,
+                            query.as_mut_ptr(),
+                        ),
+                        gst::CAT_RUST,
+                        "Parent function `decide_allocation` failed",
+                    )
+                })
+                .unwrap_or(Ok(()))
+        }
+    }
 }
 
 unsafe impl<T: BaseSrcImpl> IsSubclassable<T> for BaseSrc {
     fn class_init(klass: &mut glib::Class<Self>) {
-        <gst::Element as IsSubclassable<T>>::class_init(klass);
+        Self::parent_class_init::<T>(klass);
         let klass = klass.as_mut();
         klass.start = Some(base_src_start::<T>);
         klass.stop = Some(base_src_stop::<T>);
@@ -592,10 +630,7 @@ unsafe impl<T: BaseSrcImpl> IsSubclassable<T> for BaseSrc {
         klass.fixate = Some(base_src_fixate::<T>);
         klass.unlock = Some(base_src_unlock::<T>);
         klass.unlock_stop = Some(base_src_unlock_stop::<T>);
-    }
-
-    fn instance_init(instance: &mut glib::subclass::InitializingObject<T>) {
-        <gst::Element as IsSubclassable<T>>::instance_init(instance);
+        klass.decide_allocation = Some(base_src_decide_allocation::<T>);
     }
 }
 
@@ -603,7 +638,7 @@ unsafe extern "C" fn base_src_start<T: BaseSrcImpl>(
     ptr: *mut ffi::GstBaseSrc,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -622,7 +657,7 @@ unsafe extern "C" fn base_src_stop<T: BaseSrcImpl>(
     ptr: *mut ffi::GstBaseSrc,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -641,7 +676,7 @@ unsafe extern "C" fn base_src_is_seekable<T: BaseSrcImpl>(
     ptr: *mut ffi::GstBaseSrc,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -655,7 +690,7 @@ unsafe extern "C" fn base_src_get_size<T: BaseSrcImpl>(
     size: *mut u64,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -677,7 +712,7 @@ unsafe extern "C" fn base_src_get_times<T: BaseSrcImpl>(
     stop: *mut gst::ffi::GstClockTime,
 ) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
     let buffer = gst::BufferRef::from_ptr(buffer);
 
@@ -698,7 +733,7 @@ unsafe extern "C" fn base_src_fill<T: BaseSrcImpl>(
     buffer: *mut gst::ffi::GstBuffer,
 ) -> gst::ffi::GstFlowReturn {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
     let buffer = gst::BufferRef::from_mut_ptr(buffer);
 
@@ -716,7 +751,7 @@ unsafe extern "C" fn base_src_alloc<T: BaseSrcImpl>(
     buffer_ptr: *mut gst::ffi::GstBuffer,
 ) -> gst::ffi::GstFlowReturn {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
     // FIXME: Wrong signature in -sys bindings
     // https://gitlab.freedesktop.org/gstreamer/gstreamer-rs-sys/issues/3
@@ -734,6 +769,7 @@ unsafe extern "C" fn base_src_alloc<T: BaseSrcImpl>(
     .into_glib()
 }
 
+#[allow(clippy::needless_option_as_deref)]
 unsafe extern "C" fn base_src_create<T: BaseSrcImpl>(
     ptr: *mut ffi::GstBaseSrc,
     offset: u64,
@@ -741,7 +777,7 @@ unsafe extern "C" fn base_src_create<T: BaseSrcImpl>(
     buffer_ptr: *mut gst::ffi::GstBuffer,
 ) -> gst::ffi::GstFlowReturn {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
     // FIXME: Wrong signature in -sys bindings
     // https://gitlab.freedesktop.org/gstreamer/gstreamer-rs-sys/issues/3
@@ -825,7 +861,7 @@ unsafe extern "C" fn base_src_do_seek<T: BaseSrcImpl>(
     segment: *mut gst::ffi::GstSegment,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -843,7 +879,7 @@ unsafe extern "C" fn base_src_query<T: BaseSrcImpl>(
     query_ptr: *mut gst::ffi::GstQuery,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
     let query = gst::QueryRef::from_mut_ptr(query_ptr);
 
@@ -858,7 +894,7 @@ unsafe extern "C" fn base_src_event<T: BaseSrcImpl>(
     event_ptr: *mut gst::ffi::GstEvent,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -872,7 +908,7 @@ unsafe extern "C" fn base_src_get_caps<T: BaseSrcImpl>(
     filter: *mut gst::ffi::GstCaps,
 ) -> *mut gst::ffi::GstCaps {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
     let filter = Option::<gst::Caps>::from_glib_borrow(filter);
 
@@ -887,7 +923,7 @@ unsafe extern "C" fn base_src_negotiate<T: BaseSrcImpl>(
     ptr: *mut ffi::GstBaseSrc,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -907,7 +943,7 @@ unsafe extern "C" fn base_src_set_caps<T: BaseSrcImpl>(
     caps: *mut gst::ffi::GstCaps,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
     let caps = from_glib_borrow(caps);
 
@@ -928,7 +964,7 @@ unsafe extern "C" fn base_src_fixate<T: BaseSrcImpl>(
     caps: *mut gst::ffi::GstCaps,
 ) -> *mut gst::ffi::GstCaps {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
     let caps = from_glib_full(caps);
 
@@ -942,7 +978,7 @@ unsafe extern "C" fn base_src_unlock<T: BaseSrcImpl>(
     ptr: *mut ffi::GstBaseSrc,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -961,7 +997,7 @@ unsafe extern "C" fn base_src_unlock_stop<T: BaseSrcImpl>(
     ptr: *mut ffi::GstBaseSrc,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
 
     gst::panic_to_error!(&wrap, imp.panicked(), false, {
@@ -975,3 +1011,27 @@ unsafe extern "C" fn base_src_unlock_stop<T: BaseSrcImpl>(
     })
     .into_glib()
 }
+
+unsafe extern "C" fn base_src_decide_allocation<T: BaseSrcImpl>(
+    ptr: *mut ffi::GstBaseSrc,
+    query: *mut gst::ffi::GstQuery,
+) -> glib::ffi::gboolean {
+    let instance = &*(ptr as *mut T::Instance);
+    let imp = instance.imp();
+    let wrap: Borrowed<BaseSrc> = from_glib_borrow(ptr);
+    let query = match gst::QueryRef::from_mut_ptr(query).view_mut() {
+        gst::QueryView::Allocation(allocation) => allocation,
+        _ => unreachable!(),
+    };
+
+    gst::panic_to_error!(&wrap, imp.panicked(), false, {
+        match imp.decide_allocation(wrap.unsafe_cast_ref(), query) {
+            Ok(()) => true,
+            Err(err) => {
+                err.log_with_object(&*wrap);
+                false
+            }
+        }
+    })
+    .into_glib()
+}
